==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2107_155.zip: ronportal/ronportal/dochazka/dochazkabrowser.php <==
<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\DB\Cursor;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Kalendar;
use RonPortal\NastaveniUzivatele;

/**
 * Class RadekDochazky
 * @package Ron\RonPortal
 */
class RadekDochazky
{
    var $datum;
    var $typDne;

    /** @var $columns string[] */
    var $columns;

    //var $f_columns2; // ve zpracovane dochazce se pouzije pro barevny pruh


    function __construct()
    {
        $this->datum = 0;
        $this->typDne = 0;
        $this->columns = null;
        //$this -> f_columns2 = null;
    }
}


/**
 * Obecny seznam zaznamu na radcich po jednotlivych dnech (napr. puvodni dochazka)
 * Class DochazkaBrowser
 * @package Ron\RonPortal
 */
abstract class DochazkaBrowser extends RonPortalBrowser
{
    var $nacistSvatky;
    var $kalendar;

    /** @var  SchvaleniDochazky */
    var $schvaleni;
    var $zobrazitSchvaleni;

    /** @var  $data RadekDochazky[] */
    var $data; // nactena data - pole objektu RadekDochazky (1 zaznam pro kazdy den)
    var $zobrazitPoznamky;

    var $minimalColumnCount;
    var $pocetDatovychSloupcu; // pocet sloupcu zjisteny podle nejdelsiho radku

    /** @var int    Nastavi se automaticky po nacteni hodnot z filtru. */
    var $datumOd;
    /** @var int    Nastavi se automaticky po nacteni hodnot z filtru. */
    var $datumDo;


    /**
     * @param \Ron\DB\Database $database
     * @param string $action
     */
    function __construct($database, $action)
    {
        parent::__construct($database, $action);

        $this ->cssClass = 'browser browserdenni grid';

        $this ->fields = [];
        $this ->nacistSvatky = true;

        $this ->zobrazitSchvaleni = false;
        $this ->minimalColumnCount = 4;
    }


    /**
     * @throws \Exception
     */
    function setFilterForm()
    {
        parent::setFilterForm(); // obdobi a osoba
        $this ->filterForm ->osobyFiltr = 'PracovniPomer.ZpracovatDochazku = 1';
        $this ->filterForm ->downloadFotoOsobyUrl = '?menuid=editace&action=foto&idpp=';
        $this ->filterForm ->setUseGet(true, $this ->action);

        if (isset($_GET['zobrazitpoznamky'])) {
            $this ->zobrazitPoznamky = (int)$_GET['zobrazitpoznamky'];
            NastaveniUzivatele::setUserSettingsItem($this ->database, 'zobrazitpoznamkydochazky', $this ->zobrazitPoznamky, 0);
        } else {
            $this ->zobrazitPoznamky = NastaveniUzivatele::getUserSettingsItem($this ->database, 'zobrazitpoznamkydochazky', 0);
        }

        if ($this ->zobrazitPoznamky) {
            $this ->filterForm ->nastavLink('<a href="?menuid=' . $this ->action . '&zobrazitpoznamky=0">' . TextManager::_('Skryt_poznamky') . '</a>');
        } else {
            $this ->filterForm ->nastavLink('<a href="?menuid=' . $this ->action . '&zobrazitpoznamky=1">' . TextManager::_('Zobrazit_poznamky') . '</a>');
        }
    }


    /**
     * Potomek predefinuje tuto metodu, tak aby vracela data. Prvni sloupec musi obsahovat datum
     * @return Cursor
     */
    function nactiData()
    {
        return null;
    }


    function nactiSchvaleni()
    {
        if ($this ->filterForm) {
            $this->schvaleni = new SchvaleniDochazky();
            $this->schvaleni->load($this->database,
                $this->filterForm->vybranaOsobaRC, $this->filterForm->vybranaOsobaOsobniCislo, $this->filterForm->vybranaOsobaDatumNastupu,
                $this->datumOd, $this->datumDo);
        }
    }


    function clearData()
    {
        foreach ($this ->data as $radek) {
            $radek ->columns = [];
        }
    }


    /**
     * vytvori zaznamy pro jednotlive dny obdobi
     * @param $datumOd
     * @param $datumDo
     */
    function inicializujData($datumOd, $datumDo)
    {
        $datum = DateUtils::startOfTheDay($datumOd);

        $this ->data = [];

        // Pro kazdy den vytvori radek
        while ($datum <= $datumDo) {
            $radek = new RadekDochazky();
            $radek ->datum = $datum;

            $radek ->typDne = Kalendar::vratTypDne($this ->database, $datum, $this ->nacistSvatky, $this ->kalendar);

            $radek ->columns = [];

            $this ->data[] = $radek;

            $datum = DateUtils::startOfTheDay(DateUtils::incDay($datum, 1));
        }
    }


    /**
     * Vraci index RadekDochazky z pole $data pro zadane datum
     * @param $datum
     * @return int|string
     */
    function najdiIndexRadkuDochazky($datum)
    {
        $datum = DateUtils::startOfTheDay($datum);

        foreach ($this ->data as $i => $radek) {
            if ($datum == $radek ->datum) return $i;

            if ($datum < $radek ->datum) return -1;
        }

        return -1;
    }


    /**
     * Vraci RadekDochazky z pole $data pro zadane datum
     * @param $datum
     * @return null|RadekDochazky
     */
    function najdiRadekDochazky($datum)
    {
        $index = $this ->najdiIndexRadkuDochazky($datum);
        if ($index >= 0) return $this ->data[$index];

        return null;
    }


    /**
     * Funkce musi vratit text, ktery se zobrazi pro zadany zaznam
     * posledni urcuje, jestli je dany zaznam posledni na radku
     * @param $zaznam
     * @param $posledni
     * @return string
     */
    abstract function vratTextKZaznamu($zaznam, $posledni);


    /**
     * Zjisti maximalni delku radku z nactenych dat
     * @return int
     */
    function maximalniDelkaRadku()
    {
        $maxLength = 0;

        foreach ($this ->data as $radek) {
            if (count($radek ->columns) > $maxLength) $maxLength = count($radek ->columns);
        }

        return $maxLength;
    }


    /**
     * @param Cursor $cursor
     * @return bool
     */
    function nactiZaznamy($cursor)
    {
        $this ->clearData();

        $zaznam = $cursor ->fetch();

        while ($zaznam) {
            $dalsizaznam = $cursor ->fetch();

            $datum = DateUtils::startOfTheDay($this ->database ->stringToTimestamp($zaznam[0]));
            $i = $this ->najdiIndexRadkuDochazky($datum);

            if ($i >= 0) {
                // zjisti, jestli je aktualni zaznam posledni na radku (tj. dalsi zaznam ma jine datum)
                if ($dalsizaznam) $posledni = DateUtils::startOfTheDay($this ->database ->stringToTimestamp($dalsizaznam[0])) > $datum;
                else $posledni = true;

                // prida zaznam k radku
                // $radek -> f_columns[] = $this -> VratTextKZaznamu($zaznam, $posledni);
                $this ->data[$i] ->columns[] = $this ->vratTextKZaznamu($zaznam, $posledni);
                //$this -> f_data[$i] -> f_columns2[] = $this -> VratTextKZaznamu2($zaznam, $posledni);
            }

            $zaznam = $dalsizaznam;
        }

        return true;
    }


    /**
     * @param string $sqlFilter
     * @throws \Exception
     */
    function load($sqlFilter = '')
    {
        $this ->setFilterForm();

        if ($this ->filterForm) {
            $this ->filterForm ->loadData();
            $this ->datumOd = $this ->filterForm ->vybraneObdobiOd;
            $this ->datumDo = $this ->filterForm ->vybraneObdobiDo;
        }

        if ($this ->zobrazitSchvaleni) {
            $this->nactiSchvaleni();
        }

        $this ->inicializujData($this ->datumOd, $this ->datumDo);

        $cursor = $this ->nactiData();

        if ($cursor) {
            $this->nactiZaznamy($cursor);
        }

        $this ->pocetDatovychSloupcu = $this ->maximalniDelkaRadku();
        if ($this ->pocetDatovychSloupcu < $this ->minimalColumnCount) $this ->pocetDatovychSloupcu = $this ->minimalColumnCount; // minimalni pocet zobrazenych sloupcu
    }


    /**
     * Vraci true, pokud je prislusny den schvalen
     * @param int $datum
     * @param int $uroven
     * @return bool
     */
    function jeSchvaleno($datum, $uroven)
    {
        return $this ->schvaleni ->jeSchvaleno($datum, $uroven);
    }


    /**
     * vraci bunku se schvalenim pro dane datum
     * @param $datum
     * @param $uroven
     * @return string
     */
    function vratSloupecSchvaleni($datum, $uroven)
    {
        $classStr = 'schvaleni' . ($this ->jeSchvaleno($datum, $uroven) ? " schvaleni$uroven" : '');
        return "<td class='$classStr'></td>\n";
    }



    /**
     * @param RadekDochazky $radek
     * @return string
     */
    function vratHlavickuRadku($radek)
    {
        $result = '';

        if ($this ->zobrazitSchvaleni) {
            $result .=
                $this ->vratSloupecSchvaleni($radek ->datum, 1);
            $this ->vratSloupecSchvaleni($radek ->datum, 2);
            $this ->vratSloupecSchvaleni($radek ->datum, 3);
        }

        $result .=
            '<td class="browser_rowheader border-right">'.
            '<span class="zkratkadne">' . TextManager::_('Day_short_' . date('w', $radek ->datum)) . '</span>' .
            date('j.n.Y', $radek ->datum) . // '<span class="desktop-only">' . date('n.Y', $radek ->datum) . '</span>' .
            '</td>';

        return $result;
    }


    /**
     * @param RadekDochazky $radek
     * @param $i
     * @return string
     */
    function vratTextBunky($radek, $i)
    {
        return
            '<td>' .
            (isset($radek ->columns[$i]) ? $radek ->columns[$i] : '').
            '</td>';
    }


    function vratTextProHodnoty($indexRadku)
    {
        $result = '';

        for ($i = 0; $i < $this ->pocetDatovychSloupcu; $i++) {
            $result .= $this ->vratTextBunky($this ->data[$indexRadku], $i);
        }

        return $result;
    }


    function renderRadek($indexRadku)
    {
        echo $this ->vratHlavickuRadku($this ->data[$indexRadku]);
        echo $this ->vratTextProHodnoty($indexRadku);
    }

    function renderTableHeader()
    {
        // nic
    }


    function renderDataDochazka()
    {
        $this ->renderTableStart();

        $this ->renderTableHeader();

        echo '<tbody>';

        foreach ($this ->data as $index => $radek) {
            $cssstyle = Kalendar::getStyleForDate($this ->database, $radek ->datum, $this ->nacistSvatky, $this ->kalendar);
            if ($cssstyle) $class_str = "class='$cssstyle'";
            else $class_str = '';

            echo "<tr $class_str>";

            $this ->renderRadek($index);

            echo "</tr>\n";
        }

        $this ->renderTableFooter();

        echo "</tbody>\n";

        $this ->renderTableEnd();
    }


    function render()
    {
        $this ->renderHeader(); // nadpis a filtr

        if ($this ->filterForm  &&  !$this->kalendar) {
            $this->kalendar = Kalendar::vratKalendarOsoby($this->database,
                $this->filterForm->vybranaOsobaRC, $this->filterForm->vybranaOsobaOsobniCislo, $this->filterForm->vybranaOsobaDatumNastupu,
                $this->datumOd, $this->datumDo);
        }

        $this ->renderDataDochazka();
    }
}

==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2302_169.zip: ronportal/ronportal/dochazka/dochazkabrowser.php <==
<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\DB\Cursor;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Kalendar;
use RonPortal\NastaveniUzivatele;

/**
 * Class RadekDochazky
 */
class RadekDochazky
{
    public $datum;
    public $typDne;

    /** @public $columns string[]   Texty zobrazene v bunkach (mohou obsahovat HTML) */
    public $columns;

    /** @var string[]   Titulky jednotlivych bunek */
    public $titles;
    

    function __construct()
    {
        $this->datum = 0;
        $this->typDne = 0;
        $this->columns = [];
        $this->titles = [];
    }
}


/**
 * Obecny seznam zaznamu na radcich po jednotlivych dnech (napr. puvodni dochazka) - zaznamy se stejnym datumem se zobrazi na jednom radku
 * Class DochazkaBrowser
 */
abstract class DochazkaBrowser extends RonPortalBrowser
{
    public $nacistSvatky;
    public $kalendar;

    /** @var  SchvaleniDochazky */
    public $schvaleni;
    public $zobrazitSchvaleni;

    /** @var  $data RadekDochazky[] */
    public $data; // nactena data - pole objektu RadekDochazky (1 zaznam pro kazdy den)

    public $zobrazitFiltrPoznamky = false;

    public $minimalColumnCount;
    public $pocetDatovychSloupcu; // pocet sloupcu zjisteny podle nejdelsiho radku

    /** @var int    Nastavi se automaticky po nacteni hodnot z filtru. */
    public $datumOd;
    /** @var int    Nastavi se automaticky po nacteni hodnot z filtru. */
    public $datumDo;


    /**
     * @param \Ron\DB\Database $database
     * @param string $action
     */
    function __construct($database, $action)
    {
        parent::__construct($database, $action);

        $this ->cssClass = 'browser browserdenni grid';

        $this ->fields = [];
        $this ->nacistSvatky = true;

        $this ->zobrazitSchvaleni = false;
        $this ->minimalColumnCount = 4;

        $this ->maxPageRows = 0;    // vypne strankovani - nema smysl
    }



    /**
     * @throws \Exception
     */
    function setFilterForm()
    {
        parent::setFilterForm(); // obdobi a osoba
        $this ->filterForm ->osobyFiltr = 'PracovniPomer.ZpracovatDochazku = 1';
        $this ->filterForm ->downloadFotoOsobyUrl = '?menuid=editace&action=foto&idpp=';
        $this ->filterForm ->setUseGet(true, $this ->action);

        if ($this ->zobrazitFiltrPoznamky) {
            $this->filterForm->addFilter((new FilterFormFilter('zobrazitpoznamky'))
                ->addItem('1', TextManager::_('Zobrazit_poznamky'))
                ->addItem('0', TextManager::_('Skryt_poznamky'))
                ->setShowSelectedItem(false)
                ->setUserSettingsName('zobrazitpoznamkydochazky')
            );
        }
    }



    /**
     * Potomek predefinuje tuto metodu, tak aby vracela data. Prvni sloupec musi obsahovat datum
     * @return Cursor
     */
    function nactiData()
    {
        return null;
    }


    function nactiSchvaleni()
    {
        if ($this ->filterForm) {
            $this->schvaleni = new SchvaleniDochazky();
            $this->schvaleni->load($this->database,
                $this->filterForm->vybranaOsobaRC, $this->filterForm->vybranaOsobaOsobniCislo, $this->filterForm->vybranaOsobaDatumNastupu,
                $this->datumOd, $this->datumDo);
        }
    }


    function clearData()
    {
        foreach ($this ->data as $radek) {
            $radek ->columns = [];
            $radek ->titles = [];
        }
    }


    /**
     * Vytvori zaznamy pro jednotlive dny obdobi
     * @param $datumOd
     * @param $datumDo
     */
    function inicializujData($datumOd, $datumDo)
    {
        $datum = DateUtils::startOfTheDay($datumOd);

        $this ->data = [];

        // Pro kazdy den vytvori radek
        while ($datum <= $datumDo) {
            $radek = new RadekDochazky();
            $radek ->datum = $datum;

            $radek ->typDne = Kalendar::vratTypDne($this ->database, $datum, $this ->nacistSvatky, $this ->kalendar);

            $this ->data[] = $radek;

            $datum = DateUtils::startOfTheDay(DateUtils::incDay($datum, 1));
        }
    }


    /**
     * Vraci index RadekDochazky z pole $data pro zadane datum
     * @param $datum
     * @return int|string
     */
    function najdiIndexRadkuDochazky($datum)
    {
        $datum = DateUtils::startOfTheDay($datum);

        foreach ($this ->data as $i => $radek) {
            if ($datum == $radek ->datum) {
                return $i;
            }

            if ($datum < $radek ->datum) {
                return -1;
            }
        }

        return -1;
    }


    /**
     * Vraci RadekDochazky z pole $data pro zadane datum
     * @param $datum
     * @return null|RadekDochazky
     */
    function najdiRadekDochazky($datum)
    {
        $index = $this ->najdiIndexRadkuDochazky($datum);
        if ($index >= 0) {
            return $this ->data[$index];
        }

        return null;
    }


    /**
     * Funkce musi vratit text, ktery se zobrazi pro zadany zaznam
     * posledni urcuje, jestli je dany zaznam posledni na radku
     * @param $zaznam
     * @param $posledni
     * @return string
     */
    abstract function vratTextKZaznamu($zaznam, $posledni);


    function vratTitleKZaznamu($zaznam)
    {
        return null;
    }

    /**
     * Zjisti maximalni delku radku z nactenych dat
     * @return int
     */
    function maximalniDelkaRadku()
    {
        $maxLength = 0;

        foreach ($this ->data as $radek) {
            if (count($radek ->columns) > $maxLength) {
                $maxLength = count($radek ->columns);
            }
        }

        return $maxLength;
    }


    /**
     * @param Cursor $cursor
     * @return bool
     */
    function nactiZaznamy($cursor)
    {
        $this ->clearData();

        $zaznam = $cursor ->fetch();

        while ($zaznam) {
            $dalsiZaznam = $cursor ->fetch();

            $datum = DateUtils::startOfTheDay($this ->database ->stringToTimestamp($zaznam[0]));
            $i = $this ->najdiIndexRadkuDochazky($datum);

            if ($i >= 0) {
                // zjisti, jestli je aktualni zaznam posledni na radku (tj. dalsi zaznam ma jine datum)
                $posledni = !$dalsiZaznam || DateUtils::startOfTheDay($this->database->stringToTimestamp($dalsiZaznam[0])) > $datum;

                // prida zaznam k radku
                // $radek -> f_columns[] = $this -> VratTextKZaznamu($zaznam, $posledni);
                $this ->data[$i] ->columns[] = $this ->vratTextKZaznamu($zaznam, $posledni);
                $this ->data[$i] ->titles[] = $this ->vratTitleKZaznamu($zaznam);
                //$this -> f_data[$i] -> f_columns2[] = $this -> VratTextKZaznamu2($zaznam, $posledni);
            }

            $zaznam = $dalsiZaznam;
        }

        return true;
    }

    function loadFilterData()
    {
        parent::loadFilterData();

        $this ->datumOd = $this ->filterForm ->vybraneObdobiOd;
        $this ->datumDo = $this ->filterForm ->vybraneObdobiDo;
    }


    /**
     * @param string $sqlFilter
     * @throws \Exception
     */
    function load($sqlFilter = '')
    {
        $this ->setFilterForm();
        if ($this ->filterForm) {
            $this ->filterForm ->loadConfiguration();
        }

        $this ->loadConfiguration();

        $this ->loadFilterData();

        if ($this ->zobrazitSchvaleni) {
            $this->nactiSchvaleni();
        }

        $this ->inicializujData($this ->datumOd, $this ->datumDo);

        $cursor = $this ->nactiData();

        if ($cursor) {
            $this->nactiZaznamy($cursor);
        }

        $this ->pocetDatovychSloupcu = $this ->maximalniDelkaRadku();
        if ($this ->pocetDatovychSloupcu < $this ->minimalColumnCount) {
            // minimalni pocet zobrazenych sloupcu
            $this ->pocetDatovychSloupcu = $this ->minimalColumnCount;
        }
    }


    /**
     * Vraci true, pokud je prislusny den schvalen
     * @param int $datum
     * @param int $uroven
     * @return bool
     */
    function jeSchvaleno($datum, $uroven)
    {
        return $this ->schvaleni ->jeSchvaleno($datum, $uroven);
    }


    /**
     * vraci bunku se schvalenim pro dane datum
     * @param $datum
     * @param $uroven
     * @return string
     */
    function vratSloupecSchvaleni($datum, $uroven)
    {
        $classStr = 'schvaleni' . ($this ->jeSchvaleno($datum, $uroven) ? " schvaleni$uroven" : '');
        return "<td class='$classStr'></td>\n";
    }



    /**
     * @param RadekDochazky $radek
     * @return string
     */
    function vratHlavickuRadku($radek)
    {
        $result = '';

        if ($this ->zobrazitSchvaleni) {
            $result .=
                $this ->vratSloupecSchvaleni($radek ->datum, 1) .
                $this ->vratSloupecSchvaleni($radek ->datum, 2) .
                $this ->vratSloupecSchvaleni($radek ->datum, 3);
        }

        $result .=
            '<td class="browser_rowheader border-right">'.
            '<span class="zkratkadne">' . TextManager::_('Day_short_' . date('w', $radek ->datum)) . '</span>' .
            date('j.n.Y', $radek ->datum) . // '<span class="desktop-only">' . date('n.Y', $radek ->datum) . '</span>' .
            '</td>';

        return $result;
    }


    /**
     * @param RadekDochazky $radek
     * @param $i
     * @return string
     */
    function vratTextBunky($radek, $i)
    {
        return
            '<td' . (($radek ->titles[$i] ?? null) ? ' title="' . StringUtils::htmlTextConvert($radek ->titles[$i]) . '"' : '') . '>' .
            (isset($radek ->columns[$i]) ? $radek ->columns[$i] : '').
            '</td>';
    }


    function vratTextProHodnoty($indexRadku)
    {
        $result = '';

        for ($i = 0; $i < $this ->pocetDatovychSloupcu; $i++) {
            $result .= $this ->vratTextBunky($this ->data[$indexRadku], $i);
        }

        return $result;
    }


    function renderRadek($indexRadku)
    {
        echo $this ->vratHlavickuRadku($this ->data[$indexRadku]);
        echo $this ->vratTextProHodnoty($indexRadku);
    }

    function renderTableHeader()
    {
        // nic
    }


    function renderDataDochazka()
    {
        $this ->renderTableStart();

        $this ->renderTableHeader();

        echo '<tbody>';

        foreach ($this ->data as $index => $radek) {
            $cssstyle = Kalendar::getStyleForDate($this ->database, $radek ->datum, $this ->nacistSvatky, $this ->kalendar);
            $class_str = $cssstyle
                ? "class='$cssstyle'"
                : '';

            echo "<tr $class_str>";

            $this ->renderRadek($index);

            echo "</tr>\n";
        }

        $this ->renderTableFooter();

        echo "</tbody>\n";

        $this ->renderTableEnd();
    }

    public function renderTable()
    {
        if ($this ->filterForm  &&  !$this->kalendar) {
            $this->kalendar = Kalendar::vratKalendarOsoby($this->database,
                $this->filterForm->vybranaOsobaRC, $this->filterForm->vybranaOsobaOsobniCislo, $this->filterForm->vybranaOsobaDatumNastupu,
                $this->datumOd, $this->datumDo);
        }

        $this ->renderDataDochazka();
    }
}
